@@ -1,9 +1,9 @@
 import numpy as np
 
-from mygrad.operation_base import BroadcastableOp
+from mygrad.operation_base import Operation
 
 
-class Where(BroadcastableOp):
+class Where(Operation):
     def __call__(self, a, b, *, condition):
         self.variables = (a, b)
         self.condition = np.asarray(condition, dtype=bool)
